@@ -131,9 +131,11 @@ def group_case(tlist):
 
 def group_identifier(tlist):
     def _consume_cycle(tl, i):
+        # TODO: Usage of Wildcard token is ambivalent here.
         x = itertools.cycle((
             lambda y: (y.match(T.Punctuation, '.')
-                       or y.ttype is T.Operator),
+                       or y.ttype is T.Operator
+                       or y.ttype is T.Wildcard),
             lambda y: (y.ttype in (T.String.Symbol,
                                    T.Name,
                                    T.Wildcard,
@@ -144,15 +146,30 @@ def group_identifier(tlist):
             else:
                 raise StopIteration
 
+    def _next_token(tl, i):
+        # chooses the next token. if two tokens are found then the
+        # first is returned.
+        t1 = tl.token_next_by_type(i, (T.String.Symbol, T.Name))
+        t2 = tl.token_next_by_instance(i, sql.Function)
+        if t1 and t2:
+            i1 = tl.token_index(t1)
+            i2 = tl.token_index(t2)
+            if i1 > i2:
+                return t2
+            else:
+                return t1
+        elif t1:
+            return t1
+        else:
+            return t2
+
     # bottom up approach: group subgroups first
     [group_identifier(sgroup) for sgroup in tlist.get_sublists()
      if not isinstance(sgroup, sql.Identifier)]
 
     # real processing
     idx = 0
-    token = tlist.token_next_by_instance(idx, sql.Function)
-    if token is None:
-        token = tlist.token_next_by_type(idx, (T.String.Symbol, T.Name))
+    token = _next_token(tlist, idx)
     while token:
         identifier_tokens = [token] + list(
             _consume_cycle(tlist,
@@ -163,9 +180,7 @@ def group_identifier(tlist):
             idx = tlist.token_index(group) + 1
         else:
             idx += 1
-        token = tlist.token_next_by_instance(idx, sql.Function)
-        if token is None:
-            token = tlist.token_next_by_type(idx, (T.String.Symbol, T.Name))
+        token = _next_token(tlist, idx)
 
 
 def group_identifier_list(tlist):
@@ -173,14 +188,18 @@ def group_identifier_list(tlist):
      if not isinstance(sgroup, sql.IdentifierList)]
     idx = 0
     # Allowed list items
-    fend1_funcs = [lambda t: isinstance(t, (sql.Identifier, sql.Function)),
+    fend1_funcs = [lambda t: isinstance(t, (sql.Identifier, sql.Function,
+                                            sql.Case)),
                    lambda t: t.is_whitespace(),
                    lambda t: t.ttype == T.Name,
                    lambda t: t.ttype == T.Wildcard,
                    lambda t: t.match(T.Keyword, 'null'),
+                   lambda t: t.match(T.Keyword, 'role'),
                    lambda t: t.ttype == T.Number.Integer,
                    lambda t: t.ttype == T.String.Single,
+                   lambda t: t.ttype == T.Name.Placeholder,
                    lambda t: isinstance(t, sql.Comparison),
+                   lambda t: isinstance(t, sql.Comment),
                    ]
     tcomma = tlist.token_next_match(idx, T.Punctuation, ',')
     start = None
@@ -261,19 +280,20 @@ def group_where(tlist):
 
 
 def group_aliased(tlist):
+    clss = (sql.Identifier, sql.Function, sql.Case)
     [group_aliased(sgroup) for sgroup in tlist.get_sublists()
-     if not isinstance(sgroup, (sql.Identifier, sql.Function))]
+     if not isinstance(sgroup, clss)]
     idx = 0
-    token = tlist.token_next_by_instance(idx, (sql.Identifier, sql.Function))
+    token = tlist.token_next_by_instance(idx, clss)
     while token:
         next_ = tlist.token_next(tlist.token_index(token))
-        if next_ is not None and isinstance(next_, (sql.Identifier, sql.Function)):
+        if next_ is not None and isinstance(next_, clss):
             grp = tlist.tokens_between(token, next_)[1:]
             token.tokens.extend(grp)
             for t in grp:
                 tlist.tokens.remove(t)
         idx = tlist.token_index(token) + 1
-        token = tlist.token_next_by_instance(idx, (sql.Identifier, sql.Function))
+        token = tlist.token_next_by_instance(idx, clss)
 
 
 def group_typecasts(tlist):
@@ -297,18 +317,19 @@ def group_functions(tlist):
 
 
 def group(tlist):
-    for func in [group_parenthesis,
-                 group_functions,
-                 group_comments,
-                 group_where,
-                 group_case,
-                 group_identifier,
-                 group_typecasts,
-                 group_as,
-                 group_aliased,
-                 group_assignment,
-                 group_comparison,
-                 group_identifier_list,
-                 group_if,
-                 group_for]:
+    for func in [
+            group_comments,
+            group_parenthesis,
+            group_functions,
+            group_where,
+            group_case,
+            group_identifier,
+            group_typecasts,
+            group_as,
+            group_aliased,
+            group_assignment,
+            group_comparison,
+            group_identifier_list,
+            group_if,
+            group_for]:
         func(tlist)
