@@ -2,8 +2,13 @@
 
 import re
 
-from sqlparse import tokens as T
+from os.path import abspath, join
+
 from sqlparse import sql
+from sqlparse import tokens as T
+from sqlparse.engine import FilterStack
+from sqlparse.tokens import (Comment, Comparison, Keyword, Name, Punctuation,
+                             String, Whitespace)
 
 
 class Filter(object):
@@ -52,6 +57,109 @@ class IdentifierCaseFilter(_CaseFilter):
             yield ttype, value
 
 
+class GetComments(Filter):
+    """Get the comments from a stack"""
+    def process(self, stack, stream):
+        for token_type, value in stream:
+            if token_type in Comment:
+                yield token_type, value
+
+
+class StripComments(Filter):
+    """Strip the comments from a stack"""
+    def process(self, stack, stream):
+        for token_type, value in stream:
+            if token_type not in Comment:
+                yield token_type, value
+
+
+def StripWhitespace(stream):
+    "Strip the useless whitespaces from a stream leaving only the minimal ones"
+    last_type = None
+    has_space = False
+    ignore_group = frozenset((Comparison, Punctuation))
+
+    for token_type, value in stream:
+        # We got a previous token (not empty first ones)
+        if last_type:
+            if token_type in Whitespace:
+                has_space = True
+                continue
+
+        # Ignore first empty spaces and dot-commas
+        elif token_type in (Whitespace, Whitespace.Newline, ignore_group):
+            continue
+
+        # Yield a whitespace if it can't be ignored
+        if has_space:
+            if not ignore_group.intersection((last_type, token_type)):
+                yield Whitespace, ' '
+            has_space = False
+
+        # Yield the token and set its type for checking with the next one
+        yield token_type, value
+        last_type = token_type
+
+
+class IncludeStatement(Filter):
+    """Filter that enable a INCLUDE statement"""
+
+    def __init__(self, dirpath=".", maxRecursive=10):
+        self.dirpath = abspath(dirpath)
+        self.maxRecursive = maxRecursive
+
+        self.detected = False
+
+    def process(self, stack, stream):
+        # Run over all tokens in the stream
+        for token_type, value in stream:
+            # INCLUDE statement found, set detected mode
+            if token_type in Name and value.upper() == 'INCLUDE':
+                self.detected = True
+                continue
+
+            # INCLUDE statement was found, parse it
+            elif self.detected:
+                # Omit whitespaces
+                if token_type in Whitespace:
+                    pass
+
+                # Get path of file to include
+                path = None
+
+                if token_type in String.Symbol:
+#                if token_type in tokens.String.Symbol:
+                    path = join(self.dirpath, value[1:-1])
+
+                # Include file if path was found
+                if path:
+                    try:
+                        f = open(path)
+                        raw_sql = f.read()
+                        f.close()
+                    except IOError, err:
+                        yield Comment, u'-- IOError: %s\n' % err
+
+                    else:
+                        # Create new FilterStack to parse readed file
+                        # and add all its tokens to the main stack recursively
+                        # [ToDo] Add maximum recursive iteration value
+                        stack = FilterStack()
+                        stack.preprocess.append(IncludeStatement(self.dirpath))
+
+                        for tv in stack.run(raw_sql):
+                            yield tv
+
+                    # Set normal mode
+                    self.detected = False
+
+                # Don't include any token while in detected mode
+                continue
+
+            # Normal token
+            yield token_type, value
+
+
 # ----------------------
 # statement process
 
@@ -110,10 +218,11 @@ class StripWhitespaceFilter(Filter):
             tlist.tokens.pop(-2)
         self._stripws_default(tlist)
 
-    def process(self, stack, stmt):
-        [self.process(stack, sgroup) for sgroup in stmt.get_sublists()]
+    def process(self, stack, stmt, depth=0):
+        [self.process(stack, sgroup, depth+1)
+         for sgroup in stmt.get_sublists()]
         self._stripws(stmt)
-        if stmt.tokens[-1].is_whitespace():
+        if depth == 0 and stmt.tokens[-1].is_whitespace():
             stmt.tokens.pop(-1)
 
 
@@ -146,13 +255,14 @@ class ReindentFilter(Filter):
         split_words = ('FROM', 'JOIN$', 'AND', 'OR',
                        'GROUP', 'ORDER', 'UNION', 'VALUES',
                        'SET', 'BETWEEN')
+
         def _next_token(i):
             t = tlist.token_next_match(i, T.Keyword, split_words,
                                        regex=True)
             if t and t.value.upper() == 'BETWEEN':
-                t = _next_token(tlist.token_index(t)+1)
+                t = _next_token(tlist.token_index(t) + 1)
                 if t and t.value.upper() == 'AND':
-                    t = _next_token(tlist.token_index(t)+1)
+                    t = _next_token(tlist.token_index(t) + 1)
             return t
 
         idx = 0
@@ -215,13 +325,16 @@ class ReindentFilter(Filter):
         self.offset -= num_offset
 
     def _process_identifierlist(self, tlist):
-        identifiers = tlist.get_identifiers()
+        identifiers = list(tlist.get_identifiers())
         if len(identifiers) > 1 and not tlist.within(sql.Function):
             first = list(identifiers[0].flatten())[0]
             num_offset = self._get_offset(first) - len(first.value)
             self.offset += num_offset
             for token in identifiers[1:]:
                 tlist.insert_before(token, self.nl())
+            for token in tlist.tokens:
+                if isinstance(token, sql.Comment):
+                    tlist.insert_after(token, self.nl())
             self.offset -= num_offset
         self._process_default(tlist)
 
@@ -267,7 +380,7 @@ class ReindentFilter(Filter):
         self._process(stmt)
         if isinstance(stmt, sql.Statement):
             if self._last_stmt is not None:
-                if self._last_stmt.to_unicode().endswith('\n'):
+                if unicode(self._last_stmt).endswith('\n'):
                     nl = '\n'
                 else:
                     nl = '\n\n'
@@ -299,7 +412,7 @@ class RightMarginFilter(Filter):
                   and not token.__class__ in self.keep_together):
                 token.tokens = self._process(stack, token, token.tokens)
             else:
-                val = token.to_unicode()
+                val = unicode(token)
                 if len(self.line) + len(val) > self.width:
                     match = re.search('^ +', self.line)
                     if match is not None:
@@ -316,19 +429,78 @@ class RightMarginFilter(Filter):
         group.tokens = self._process(stack, group, group.tokens)
 
 
+class ColumnsSelect(Filter):
+    """Get the columns names of a SELECT query"""
+    def process(self, stack, stream):
+        mode = 0
+        oldValue = ""
+        parenthesis = 0
+
+        for token_type, value in stream:
+            # Ignore comments
+            if token_type in Comment:
+                continue
+
+            # We have not detected a SELECT statement
+            if mode == 0:
+                if token_type in Keyword and value == 'SELECT':
+                    mode = 1
+
+            # We have detected a SELECT statement
+            elif mode == 1:
+                if value == 'FROM':
+                    if oldValue:
+                        yield oldValue
+
+                    mode = 3    # Columns have been checked
+
+                elif value == 'AS':
+                    oldValue = ""
+                    mode = 2
+
+                elif (token_type == Punctuation
+                      and value == ',' and not parenthesis):
+                    if oldValue:
+                        yield oldValue
+                    oldValue = ""
+
+                elif token_type not in Whitespace:
+                    if value == '(':
+                        parenthesis += 1
+                    elif value == ')':
+                        parenthesis -= 1
+
+                    oldValue += value
+
+            # We are processing an AS keyword
+            elif mode == 2:
+                # We check also for Keywords because a bug in SQLParse
+                if token_type == Name or token_type == Keyword:
+                    yield value
+                    mode = 1
+
+
 # ---------------------------
 # postprocess
 
 class SerializerUnicode(Filter):
 
     def process(self, stack, stmt):
-        raw = stmt.to_unicode()
+        raw = unicode(stmt)
         add_nl = raw.endswith('\n')
         res = '\n'.join(line.rstrip() for line in raw.splitlines())
         if add_nl:
             res += '\n'
         return res
 
+def Tokens2Unicode(stream):
+    result = ""
+
+    for _, value in stream:
+        result += unicode(value)
+
+    return result
+
 
 class OutputPythonFilter(Filter):
 
@@ -374,7 +546,7 @@ class OutputPythonFilter(Filter):
             varname = '%s%d' % (self.varname, self.cnt)
         else:
             varname = self.varname
-        has_nl = len(stmt.to_unicode().strip().splitlines()) > 1
+        has_nl = len(unicode(stmt).strip().splitlines()) > 1
         stmt.tokens = self._process(stmt.tokens, varname, self.cnt, has_nl)
         return stmt
 
@@ -422,3 +594,24 @@ class OutputPHPFilter(Filter):
             varname = self.varname
         stmt.tokens = tuple(self._process(stmt.tokens, varname))
         return stmt
+
+
+class Limit(Filter):
+    """Get the LIMIT of a query.
+
+    If not defined, return -1 (SQL specification for no LIMIT query)
+    """
+    def process(self, stack, stream):
+        index = 7
+        stream = list(stream)
+        stream.reverse()
+
+        # Run over all tokens in the stream from the end
+        for token_type, value in stream:
+            index -= 1
+
+#            if index and token_type in Keyword:
+            if index and token_type in Keyword and value == 'LIMIT':
+                return stream[4 - index][1]
+
+        return -1
