@@ -177,14 +177,12 @@ class Lexer(object):
             (r"´(´´|[^´])*´", tokens.Name),
             (r'\$([a-zA-Z_][a-zA-Z0-9_]*)?\$', tokens.Name.Builtin),
             (r'\?{1}', tokens.Name.Placeholder),
-            (r'[$:?%][a-zA-Z0-9_]+[^$:?%]?', tokens.Name.Placeholder),
+            (r'[$:?%][a-zA-Z0-9_]+', tokens.Name.Placeholder),
             (r'@[a-zA-Z_][a-zA-Z0-9_]+', tokens.Name),
             (r'[a-zA-Z_][a-zA-Z0-9_]*(?=[.(])', tokens.Name),  # see issue39
-            (r'[<>=~!]+', tokens.Operator.Comparison),
-            (r'[+/@#%^&|`?^-]+', tokens.Operator),
-            (r'0x[0-9a-fA-F]+', tokens.Number.Hexadecimal),
-            (r'[0-9]*\.[0-9]+', tokens.Number.Float),
-            (r'[0-9]+', tokens.Number.Integer),
+            (r'[-]?0x[0-9a-fA-F]+', tokens.Number.Hexadecimal),
+            (r'[-]?[0-9]*\.[0-9]+', tokens.Number.Float),
+            (r'[-]?[0-9]+', tokens.Number.Integer),
             # TODO: Backslash escapes?
             (r"(''|'.*?[^\\]')", tokens.String.Single),
             # not a real string literal in ANSI SQL:
@@ -194,8 +192,11 @@ class Lexer(object):
             (r'END( IF| LOOP)?\b', tokens.Keyword),
             (r'NOT NULL\b', tokens.Keyword),
             (r'CREATE( OR REPLACE)?\b', tokens.Keyword.DDL),
+            (r'(?<=\.)[a-zA-Z_][a-zA-Z0-9_]*', tokens.Name),
             (r'[a-zA-Z_][a-zA-Z0-9_]*', is_keyword),
             (r'[;:()\[\],\.]', tokens.Punctuation),
+            (r'[<>=~!]+', tokens.Operator.Comparison),
+            (r'[+/@#%^&|`?^-]+', tokens.Operator),
         ],
         'multiline-comments': [
             (r'/\*', tokens.Comment.Multiline, 'multiline-comments'),
@@ -230,15 +231,6 @@ class Lexer(object):
                         text = text[len(u'\ufeff'):]
                 except UnicodeDecodeError:
                     text = text.decode('latin1')
-            elif self.encoding == 'chardet':
-                try:
-                    import chardet
-                except ImportError:
-                    raise ImportError('To enable chardet encoding guessing, '
-                                      'please install the chardet library '
-                                      'from http://chardet.feedparser.org/')
-                enc = chardet.detect(text)
-                text = text.decode(enc['encoding'])
             else:
                 text = text.decode(self.encoding)
         if self.stripall:
@@ -265,7 +257,7 @@ class Lexer(object):
         ``stack`` is the inital stack (default: ``['root']``)
         """
         pos = 0
-        tokendefs = self._tokens
+        tokendefs = self._tokens  # see __call__, pylint:disable=E1101
         statestack = list(stack)
         statetokens = tokendefs[statestack[-1]]
         known_names = {}
