@@ -31,10 +31,14 @@ class Token(object):
                                           short, id(self))
 
     def __unicode__(self):
+        """Returns a unicode representation of this object."""
         return self.value or ''
 
     def to_unicode(self):
-        """Returns a unicode representation of this object."""
+        """Returns a unicode representation of this object.
+        
+        @deprecated: please use __unicode__()
+        """
         return unicode(self)
 
     def _get_repr_name(self):
@@ -133,20 +137,17 @@ class TokenList(Token):
         if tokens is None:
             tokens = []
         self.tokens = tokens
-        Token.__init__(self, None, None)
+        Token.__init__(self, None, unicode(self))
 
     def __unicode__(self):
         return ''.join(unicode(x) for x in self.flatten())
 
-    def __str__(self):
-        return unicode(self).encode('utf-8')
-
     def _get_repr_name(self):
         return self.__class__.__name__
 
     def _pprint_tree(self, max_depth=None, depth=0):
         """Pretty-print the object tree."""
-        indent = ' '*(depth*2)
+        indent = ' ' * (depth * 2)
         for idx, token in enumerate(self.tokens):
             if token.is_group():
                 pre = ' +-'
@@ -156,7 +157,7 @@ class TokenList(Token):
                                         token._get_repr_name(),
                                         token._get_repr_value())
             if (token.is_group() and (max_depth is None or depth < max_depth)):
-                token._pprint_tree(max_depth, depth+1)
+                token._pprint_tree(max_depth, depth + 1)
 
     def flatten(self):
         """Generator yielding ungrouped tokens.
@@ -174,7 +175,10 @@ class TokenList(Token):
         return True
 
     def get_sublists(self):
-        return [x for x in self.tokens if isinstance(x, TokenList)]
+#        return [x for x in self.tokens if isinstance(x, TokenList)]
+        for x in self.tokens:
+            if isinstance(x, TokenList):
+                yield x
 
     @property
     def _groupable_tokens(self):
@@ -190,7 +194,6 @@ class TokenList(Token):
             if ignore_whitespace and token.is_whitespace():
                 continue
             return token
-        return None
 
     def token_next_by_instance(self, idx, clss):
         """Returns the next token matching a class.
@@ -202,30 +205,31 @@ class TokenList(Token):
         """
         if isinstance(clss, (list, tuple)):
             clss = (clss,)
+
         if isinstance(clss, tuple):
             clss = tuple(clss)
+
         for token in self.tokens[idx:]:
             if isinstance(token, clss):
                 return token
-        return None
 
     def token_next_by_type(self, idx, ttypes):
         """Returns next matching token by it's token type."""
         if not isinstance(ttypes, (list, tuple)):
             ttypes = [ttypes]
+
         for token in self.tokens[idx:]:
             if token.ttype in ttypes:
                 return token
-        return None
 
     def token_next_match(self, idx, ttype, value, regex=False):
         """Returns next token where it's ``match`` method returns ``True``."""
         if not isinstance(idx, int):
             idx = self.token_index(idx)
+
         for token in self.tokens[idx:]:
             if token.match(ttype, value, regex):
                 return token
-        return None
 
     def token_not_matching(self, idx, funcs):
         for token in self.tokens[idx:]:
@@ -234,16 +238,15 @@ class TokenList(Token):
                 if func(token):
                     passed = True
                     break
+
             if not passed:
                 return token
-        return None
 
     def token_matching(self, idx, funcs):
         for token in self.tokens[idx:]:
             for i, func in enumerate(funcs):
                 if func(token):
                     return token
-        return None
 
     def token_prev(self, idx, skip_ws=True):
         """Returns the previous token relative to *idx*.
@@ -253,8 +256,10 @@ class TokenList(Token):
         """
         if idx is None:
             return None
+
         if not isinstance(idx, int):
             idx = self.token_index(idx)
+
         while idx != 0:
             idx -= 1
             if self.tokens[idx].is_whitespace() and skip_ws:
@@ -269,8 +274,10 @@ class TokenList(Token):
         """
         if idx is None:
             return None
+
         if not isinstance(idx, int):
             idx = self.token_index(idx)
+
         while idx < len(self.tokens) - 1:
             idx += 1
             if self.tokens[idx].is_whitespace() and skip_ws:
@@ -315,37 +322,13 @@ class TokenList(Token):
         """Inserts *token* before *where*."""
         self.tokens.insert(self.token_index(where), token)
 
-
-class Statement(TokenList):
-    """Represents a SQL statement."""
-
-    __slots__ = ('value', 'ttype', 'tokens')
-
-    def get_type(self):
-        """Returns the type of a statement.
-
-        The returned value is a string holding an upper-cased reprint of
-        the first DML or DDL keyword. If the first token in this group
-        isn't a DML or DDL keyword "UNKNOWN" is returned.
-        """
-        first_token = self.token_first()
-        if first_token is None:
-            # An "empty" statement that either has not tokens at all
-            # or only whitespace tokens.
-            return 'UNKNOWN'
-        elif first_token.ttype in (T.Keyword.DML, T.Keyword.DDL):
-            return first_token.value.upper()
+    def insert_after(self, where, token):
+        """Inserts *token* after *where*."""
+        next_token = self.token_next(where)
+        if next_token is None:
+            self.tokens.append(token)
         else:
-            return 'UNKNOWN'
-
-
-class Identifier(TokenList):
-    """Represents an identifier.
-
-    Identifiers may have aliases or typecasts.
-    """
-
-    __slots__ = ('value', 'ttype', 'tokens')
+            self.tokens.insert(self.token_index(next_token), token)
 
     def has_alias(self):
         """Returns ``True`` if an alias is present."""
@@ -359,14 +342,13 @@ class Identifier(TokenList):
             if alias is None:
                 return None
         else:
-            next_ = self.token_next(0)
-            if next_ is None or not isinstance(next_, Identifier):
+            next_ = self.token_next_by_instance(0, Identifier)
+            if next_ is None:
                 return None
             alias = next_
         if isinstance(alias, Identifier):
             return alias.get_name()
-        else:
-            return alias.to_unicode()
+        return unicode(alias)
 
     def get_name(self):
         """Returns the name of this identifier.
@@ -393,6 +375,39 @@ class Identifier(TokenList):
                 return None
             return next_.value
 
+
+class Statement(TokenList):
+    """Represents a SQL statement."""
+
+    __slots__ = ('value', 'ttype', 'tokens')
+
+    def get_type(self):
+        """Returns the type of a statement.
+
+        The returned value is a string holding an upper-cased reprint of
+        the first DML or DDL keyword. If the first token in this group
+        isn't a DML or DDL keyword "UNKNOWN" is returned.
+        """
+        first_token = self.token_first()
+        if first_token is None:
+            # An "empty" statement that either has not tokens at all
+            # or only whitespace tokens.
+            return 'UNKNOWN'
+
+        elif first_token.ttype in (T.Keyword.DML, T.Keyword.DDL):
+            return first_token.value.upper()
+
+        return 'UNKNOWN'
+
+
+class Identifier(TokenList):
+    """Represents an identifier.
+
+    Identifiers may have aliases or typecasts.
+    """
+
+    __slots__ = ('value', 'ttype', 'tokens')
+
     def get_parent_name(self):
         """Return name of the parent object if any.
 
@@ -419,7 +434,7 @@ class Identifier(TokenList):
         next_ = self.token_next(self.token_index(marker), False)
         if next_ is None:
             return None
-        return next_.to_unicode()
+        return unicode(next_)
 
 
 class IdentifierList(TokenList):
@@ -430,10 +445,11 @@ class IdentifierList(TokenList):
     def get_identifiers(self):
         """Returns the identifiers.
 
-        Whitespaces and punctuations are not included in this list.
+        Whitespaces and punctuations are not included in this generator.
         """
-        return [x for x in self.tokens
-                if not x.is_whitespace() and not x.match(T.Punctuation, ',')]
+        for x in self.tokens:
+            if not x.is_whitespace() and not x.match(T.Punctuation, ','):
+                yield x
 
 
 class Parenthesis(TokenList):
@@ -485,33 +501,43 @@ class Case(TokenList):
 
         If an ELSE exists condition is None.
         """
+        CONDITION = 1
+        VALUE = 2
+
         ret = []
-        in_value = False
-        in_condition = True
+        mode = CONDITION
+
         for token in self.tokens:
+            # Set mode from the current statement
             if token.match(T.Keyword, 'CASE'):
                 continue
+
             elif token.match(T.Keyword, 'WHEN'):
                 ret.append(([], []))
-                in_condition = True
-                in_value = False
+                mode = CONDITION
+
+            elif token.match(T.Keyword, 'THEN'):
+                mode = VALUE
+
             elif token.match(T.Keyword, 'ELSE'):
                 ret.append((None, []))
-                in_condition = False
-                in_value = True
-            elif token.match(T.Keyword, 'THEN'):
-                in_condition = False
-                in_value = True
+                mode = VALUE
+
             elif token.match(T.Keyword, 'END'):
-                in_condition = False
-                in_value = False
-            if (in_condition or in_value) and not ret:
-                # First condition withou preceding WHEN
+                mode = None
+
+            # First condition without preceding WHEN
+            if mode and not ret:
                 ret.append(([], []))
-            if in_condition:
+
+            # Append token depending of the current mode
+            if mode == CONDITION:
                 ret[-1][0].append(token)
-            elif in_value:
+
+            elif mode == VALUE:
                 ret[-1][1].append(token)
+
+        # Return cases list
         return ret
 
 
