@@ -68,6 +68,12 @@ class TestFormat(TestCaseBase):
         self.assertRaises(sqlparse.SQLParseError, sqlparse.format, s,
                           strip_whitespace=None)
 
+    def test_preserve_ws(self):
+        # preserve at least one whitespace after subgroups
+        f = lambda sql: sqlparse.format(sql, strip_whitespace=True)
+        s = 'select\n* /* foo */  from bar '
+        self.ndiffAssertEqual(f(s), 'select * /* foo */ from bar')
+
     def test_outputformat(self):
         sql = 'select * from foo;'
         self.assertRaises(sqlparse.SQLParseError, sqlparse.format, sql,
@@ -228,7 +234,12 @@ class TestFormatReindent(TestCaseBase):
                                                '       foo,',
                                                '       bar']))
 
-
+    def test_identifier_and_functions(self):  # issue45
+        f = lambda sql: sqlparse.format(sql, reindent=True)
+        s = 'select foo.bar, nvl(1) from dual'
+        self.ndiffAssertEqual(f(s), '\n'.join(['select foo.bar,',
+                                               '       nvl(1)',
+                                               'from dual']))
 
 
 class TestOutputFormat(TestCaseBase):
