@@ -10,7 +10,7 @@ from tests.utils import TestCaseBase
 class TestGrouping(TestCaseBase):
 
     def test_parenthesis(self):
-        s ='select (select (x3) x2) and (y2) bar'
+        s = 'select (select (x3) x2) and (y2) bar'
         parsed = sqlparse.parse(s)[0]
         self.ndiffAssertEqual(s, str(parsed))
         self.assertEqual(len(parsed.tokens), 9)
@@ -39,18 +39,18 @@ class TestGrouping(TestCaseBase):
     def test_identifiers(self):
         s = 'select foo.bar from "myscheme"."table" where fail. order'
         parsed = sqlparse.parse(s)[0]
-        self.ndiffAssertEqual(s, parsed.to_unicode())
+        self.ndiffAssertEqual(s, unicode(parsed))
         self.assert_(isinstance(parsed.tokens[2], sql.Identifier))
         self.assert_(isinstance(parsed.tokens[6], sql.Identifier))
         self.assert_(isinstance(parsed.tokens[8], sql.Where))
         s = 'select * from foo where foo.id = 1'
         parsed = sqlparse.parse(s)[0]
-        self.ndiffAssertEqual(s, parsed.to_unicode())
+        self.ndiffAssertEqual(s, unicode(parsed))
         self.assert_(isinstance(parsed.tokens[-1].tokens[-1].tokens[0],
                                 sql.Identifier))
         s = 'select * from (select "foo"."id" from foo)'
         parsed = sqlparse.parse(s)[0]
-        self.ndiffAssertEqual(s, parsed.to_unicode())
+        self.ndiffAssertEqual(s, unicode(parsed))
         self.assert_(isinstance(parsed.tokens[-1].tokens[3], sql.Identifier))
 
     def test_identifier_wildcard(self):
@@ -117,46 +117,51 @@ class TestGrouping(TestCaseBase):
     def test_where(self):
         s = 'select * from foo where bar = 1 order by id desc'
         p = sqlparse.parse(s)[0]
-        self.ndiffAssertEqual(s, p.to_unicode())
+        self.ndiffAssertEqual(s, unicode(p))
         self.assertTrue(len(p.tokens), 16)
         s = 'select x from (select y from foo where bar = 1) z'
         p = sqlparse.parse(s)[0]
-        self.ndiffAssertEqual(s, p.to_unicode())
+        self.ndiffAssertEqual(s, unicode(p))
         self.assertTrue(isinstance(p.tokens[-3].tokens[-2], sql.Where))
 
     def test_typecast(self):
         s = 'select foo::integer from bar'
         p = sqlparse.parse(s)[0]
-        self.ndiffAssertEqual(s, p.to_unicode())
+        self.ndiffAssertEqual(s, unicode(p))
         self.assertEqual(p.tokens[2].get_typecast(), 'integer')
         self.assertEqual(p.tokens[2].get_name(), 'foo')
         s = 'select (current_database())::information_schema.sql_identifier'
         p = sqlparse.parse(s)[0]
-        self.ndiffAssertEqual(s, p.to_unicode())
+        self.ndiffAssertEqual(s, unicode(p))
         self.assertEqual(p.tokens[2].get_typecast(),
                          'information_schema.sql_identifier')
 
     def test_alias(self):
         s = 'select foo as bar from mytable'
         p = sqlparse.parse(s)[0]
-        self.ndiffAssertEqual(s, p.to_unicode())
+        self.ndiffAssertEqual(s, unicode(p))
         self.assertEqual(p.tokens[2].get_real_name(), 'foo')
         self.assertEqual(p.tokens[2].get_alias(), 'bar')
         s = 'select foo from mytable t1'
         p = sqlparse.parse(s)[0]
-        self.ndiffAssertEqual(s, p.to_unicode())
+        self.ndiffAssertEqual(s, unicode(p))
         self.assertEqual(p.tokens[6].get_real_name(), 'mytable')
         self.assertEqual(p.tokens[6].get_alias(), 't1')
         s = 'select foo::integer as bar from mytable'
         p = sqlparse.parse(s)[0]
-        self.ndiffAssertEqual(s, p.to_unicode())
+        self.ndiffAssertEqual(s, unicode(p))
         self.assertEqual(p.tokens[2].get_alias(), 'bar')
         s = ('SELECT DISTINCT '
              '(current_database())::information_schema.sql_identifier AS view')
         p = sqlparse.parse(s)[0]
-        self.ndiffAssertEqual(s, p.to_unicode())
+        self.ndiffAssertEqual(s, unicode(p))
         self.assertEqual(p.tokens[4].get_alias(), 'view')
 
+    def test_alias_case(self):  # see issue46
+        p = sqlparse.parse('CASE WHEN 1 THEN 2 ELSE 3 END foo')[0]
+        self.assertEqual(len(p.tokens), 1)
+        self.assertEqual(p.tokens[0].get_alias(), 'foo')
+
     def test_idlist_function(self):  # see issue10 too
         p = sqlparse.parse('foo(1) x, bar')[0]
         self.assert_(isinstance(p.tokens[0], sql.IdentifierList))
@@ -176,7 +181,7 @@ class TestGrouping(TestCaseBase):
         self.assert_(isinstance(p.tokens[0], sql.Function))
         p = sqlparse.parse('foo(null, bar)')[0]
         self.assert_(isinstance(p.tokens[0], sql.Function))
-        self.assertEqual(len(p.tokens[0].get_parameters()), 2)
+        self.assertEqual(len(list(p.tokens[0].get_parameters())), 2)
 
 
 class TestStatement(TestCaseBase):
